function [SurfaceData,SurfaceNormal,Vd] = DropletSurface_EiF(Hch,ar,kf,kb,Ld_Wch,d_h,kgf,kgb,Ndrop,Flag_ShowFigure,Flag_ComputeVolume)
%
% DropletSurface_EiF    
%
%           returns the surface points, the corresponding normal vectors 
%        	and the enclosed volume of a Taylor droplet surface. 
%
%       	as pubilshed            Miener, U., Helmers, T., Lindken, R. et al. 
%                               	Exp Fluids (2019) 60: 75. 
%                               	https://doi.org/10.1007/s00348-019-2719-0
%
%   [SurfaceData,SurfaceNormal,Vd] = DropletSurface_EiF(Hch,ar,kf,kb,Ld_Wch,d_h,kgf,kgb,Ndrop,Flag_ShowFigure,Flag_ComputeVolume)
%
%   returns             SurfaceData             (n,n,3) matrix containing n x n points along the 3 cartesian axis x,y,z
%                                               The cartesian origin is set to the front tip of the Taylor droplet. 
%                       SurfaceNormal           (n,n,3) matrix containing n x n normal vector for the points given in SurfaceData 
%                       Vd                      [OPTIONAL] caluates the volume of the Taylor droplet inside the surface using
%                                               Vd uses the <<delaunayTriangulation>> to compute the volume
%
%   input parameters    Hch                     Height of the rectangular micro channel: The height sets the dimensions of the droplet coordinates.
%                       ar                      Aspect ratio of the rectangular micro channel (ar = Wch/Hch >= 1). 
%
%                       kf                      Cap deformation ratio of the Taylor droplet front (kf >=1)      
%                                               =>  flow related elongation of the droplet front.
%                       kb                      Cap deformation ratio of the Taylor droplet back (kf <=1)       
%                                               =>  flow related elongation of the droplet back.
%
%                       Ld_Wch                  Length ratio of the Taylor droplet (Ld_Wch = Ld / Wch >= 1).
%                       d_h                     dimensionless wall film thickness (e.g. DOI:10.1002/ceat.201100643)
%
%                       kgf                     dimensionless front gutter radius of a Taylor droplet 
%                                               -> see FUNCTION_DynamicGutterRadius_EiF
%                       kgb                     dimensionless back  gutter radius of a Taylor droplet 
%                                               -> see FUNCTION_DynamicGutterRadius_EiF
%
%                       Ndrop                   Number of supporting points of the Taylor droplet surface => n
%
%                       Flag_ShowFigure         Set Flag to 1 to recieve a 3D plot of the Taylor droplate surface
%                       Flag_ComputeVolume      Set Flag to 1 to compute and return the Taylor droplet volume
%
%





%%

if not(exist('Flag_ComputeVolume','var'))
    Flag_ComputeVolume = 1;
 
else
    if Flag_ComputeVolume==0
        Vd = nan;
    end
end

if not(exist('Flag_ShowFigure','var'))
    Flag_ShowFigure   = 0;   
end


%%
N               = floor(Ndrop/(5*2))*(5*2);      % Number of Supporting points needs to be devidable by 5 and 2 

Nx1             = round(3/10*N);
Nx2             = N-Nx1;
Nz              = round(8/10*N);

% The reference length for the calculation is the channel height.
% the orientation of the droplet is not important for the shape nor the
% volume. Thus, in case ar < 1 the channel geometry is permuted. 
% 
% es ist jedoch geometrisch nicht wichtig fr das Volumen 
% wie der Tropfen orientiert ist.
% Daher wird bei aspectRatio < 1 die Kanalgeometrie permuttiert 
% und die Berechnung immer in derselben Form durchgefhrt.
Wch             = Hch * ar;    % [m] Kanalbreite


dFilm           = min(Hch,Wch).*d_h; % Filmthickness
H               = Hch - 2*dFilm;
W               = Wch - 2*dFilm;

Ld              = max([Wch Hch]) .* Ld_Wch; % Droplet length

Rgf             = kgf * H;
Rgb             = kgb * W;

% The gutter radius is valid for a minimum Taylor droplet length.
% Below that minimum lengththere is not enough volume to properly occupy
% the channel cross section.
Ld_min          = ((W/2*kf) +  (H/2-Rgf)) + ((W/2*kb) +  (H/2-Rgb));
lengthRatio_min = Ld_min / Wch;


if Ld_Wch >= lengthRatio_min
    % In case the droplet length is greater than the minimum length, no scaling
    % of the gutter radius is needed.
    scaleRg = 1;
else
    % In case the droplet length is smaller than the minimum length, the
    % gutter radius needs to rise, since not enough droplet volume is 
    % available to fill the theoretical gutter region 
    scaleRg = ar .* (1/(kgf+kgb))  * ((kf+kb)/2   +1/ar   -Ld_Wch );
end
Rgf = Rgf * scaleRg;
Rgb = Rgb * scaleRg;

%%
% Ellipse front at the channel top
b_front_top    	= (W/2-Rgf);
a_front_top     = kf*b_front_top;

xm_front_top    = -(H/2*kf + a_front_top);
ym_front_top    = 0;

% Ellipse back at the channel top
b_back_top      = (W/2-Rgb);
a_back_top      = kb*b_back_top;

xm_back_top     = -(Ld- (H/2*kb+a_back_top ));
ym_back_top     = 0;

% Calculation of the tagantial boundary points for the droplet front and the back 
[phi_f1, phi_b2]  = filmTangentenWinkel(a_front_top,b_front_top,xm_front_top,ym_front_top,a_back_top,b_back_top,xm_back_top,ym_back_top);

xpf1            = a_front_top   .* cos(phi_f1) + xm_front_top;
ypf1            = b_front_top   .* sin(phi_f1) + ym_front_top;
zpf1            = H/2;

phi             = linspace(0,phi_f1,N);
x1              = a_front_top .* cos(phi) + xm_front_top;
y1              = b_front_top .* sin(phi) + ym_front_top;
z1              = ones(size(x1)).* (zpf1);

xpb2            = a_back_top    .* cos(phi_b2) + xm_back_top;
ypb2            = b_back_top    .* sin(phi_b2) + ym_back_top;

phi             = linspace(phi_b2,1*pi,N);
x2              = a_back_top .* cos(phi) + xm_back_top;
y2              = b_back_top .* sin(phi) + ym_back_top;
z2              = z1;

%%

% Ellipse front left side wall
c_front_left    = (H/2-Rgf);
a_front_left    = kf*c_front_left;

xm_front_left   = xm_front_top; 
zm_front_left   = 0;

% Ellipse back left side wall
c_back_left     = (H/2-Rgb);
a_back_left     = kb*c_back_left;

xm_back_left    = xm_back_top; 
zm_back_left    = 0;

% Calculation of the tagantial boundary points for the droplet front and the back 
[phi_f3, phi_b4]  = filmTangentenWinkel(a_front_left,c_front_left,xm_front_left,zm_front_left,a_back_left,c_back_left,xm_back_left,zm_back_left);

xpfl3         	= a_front_left   .* cos(phi_f3) + xm_front_left;
ypfl3         	= W/2;
zpfl3          	= c_front_left   .* sin(phi_f3) + zm_front_left;

phi             = linspace(0,phi_f3,N);
x3              = a_front_left .* cos(phi) + xm_front_left;
y3              = ones(size(x1)).* ypfl3 ;
z3              = c_front_left .* sin(phi) + zm_front_left;

xpbl4         	= a_back_left    .* cos(phi_b4) + xm_back_left;
zpbl4       	= c_back_left    .* sin(phi_b4) + zm_back_left;

phi             = linspace(phi_b4,1*pi,N);
x4              = a_back_left .* cos(phi) + xm_back_left;
y4              = y3 ;
z4              = c_back_left .* sin(phi) + zm_back_left;

%% Calculation of the front gutter line
xmgf = xm_front_top;
ymgf = W/2 - Rgf;
zmgf = H/2 - Rgf;

alpha   = linspace(pi/2,0,Nz);

xgf     = ones(1,Nz).* xmgf;
ygf     = ymgf + Rgf .* cos(pi/2-alpha);
zgf     = zmgf + Rgf .* sin(pi/2-alpha);


%% Calculation of the back gutter line
xmgb = xm_back_top;
ymgb = W/2 - Rgb;
zmgb = H/2 - Rgb;

alpha   = linspace(pi/2,0,Nz);

xgb     = ones(1,Nz).* xmgb;
ygb     = ymgb + Rgb .* cos(pi/2-alpha);
zgb     = zmgb + Rgb .* sin(pi/2-alpha);


%% Calculation of the front contour ellipse xy

b_front_xy      = (W/2);
a_front_xy      = kf*b_front_xy;

xm_front_xy     = -(W/2*kf );
ym_front_xy     = 0;
zm_front_xy     = 0;

gamma = linspace(0,pi/2,N);

x_front_xy = xm_front_xy + a_front_xy .* cos(gamma);
y_front_xy = ym_front_xy + b_front_xy .* sin(gamma);
z_front_xy = ones(size(x_front_xy)).*zm_front_xy;

%% Calculation of the back contour ellipse xy

b_back_xy      = (W/2);
a_back_xy      = kb*b_back_xy;

xm_back_xy     = -(Ld-a_back_xy  );
ym_back_xy     = 0;
zm_back_xy     = 0;

gamma = linspace(pi/2,pi,N);

x_back_xy = xm_back_xy + a_back_xy .* cos(gamma);
y_back_xy = ym_back_xy + b_back_xy .* sin(gamma);
z_back_xy = ones(size(x_back_xy)).*zm_back_xy;

%% Calculation of the front contour ellipse xz

c_front_xz      = (H/2);
a_front_xz      = a_front_xy + a_front_left - a_front_top;

xm_front_xz     = -(a_front_xz);
ym_front_xz     = 0;
zm_front_xz     = 0;

gamma = linspace(0,pi/2,N);

x_front_xz = xm_front_xz + a_front_xz .* cos(gamma);
y_front_xz = ones(size(x_front_xz)).*ym_front_xz;
z_front_xz = zm_front_xz + c_front_xz .* sin(gamma);

%% Calculation of the back contour ellipse xz

c_back_xz      = (H/2);
a_back_xz      = a_back_xy + a_back_left - a_back_top;

xm_back_xz     = -(Ld-a_back_xz);
ym_back_xz     = 0;
zm_back_xz     = 0;

gamma = linspace(pi/2,pi,N);

x_back_xz = xm_back_xz + a_back_xz .* cos(gamma);
y_back_xz = ones(size(x_back_xz)).*ym_back_xz;
z_back_xz = zm_back_xz + c_back_xz .* sin(gamma);




%% Calculation of the front ellipse area 
xmfc            = xm_front_top;
ymfc            = 0;
zmfc            = 0;
    
a_front_cap     = abs(xmfc);
b_front_cap     = W/2;
c_front_cap     = H/2;
    
% Calculation of the back ellipse area 
xmbc            = -(Ld-W/2*kb);
ymbc            = 0;
zmbc            = 0;
    
a_back_cap      = abs(W/2*kb);
b_back_cap      = b_front_cap;
c_back_cap      = c_front_cap;
    
theta           = linspace(0,1*pi/2,N);
  
xcf             = zeros(N,N).*NaN;
ycf             = zeros(N,N).*NaN;
zcf             = zeros(N,N).*NaN;



% alpha     angle around the x-axis => yz-plane
% beta      angle around the y-axis => xz-plane
% gamma     angle around the z-axis => xy-plane

% theta     elevation angle for the polar coordinates of the ellipsoids
% phi       planar angle    for the polar coordinates of the ellipsoids

%% Calculation: points of the top film area

n = (N-2*Nx1)/2;
% front ellipse top film
bf = linspace(b_front_top,0.001.*b_front_top,n);
af = linspace(a_front_top,0.001.*a_front_top,n);

xm_front_top    = -(H/2*kf + a_front_top);
ym_front_top    = 0;

% back ellipse top film
bb = linspace(b_back_top,0.001.*b_back_top,n);
ab = linspace(a_back_top,0.001.*a_back_top,n);

xm_back_top     = -(Ld- (H/2*kb+a_back_top ));
ym_back_top     = 0;

X = ones(N,N).*NaN;
Y = ones(N,N).*NaN;
Z = ones(N,N).*NaN;

% angular deviation, to avoid dublicate points at the surface joints
% dphi <= 0; duplicated points are tolrated.
% dphi = 0.1 duplicated points are avoided.
dphi = 0.0; 

for i = Nz+1: N
    
    j = i-(Nz);
    
    % Calculation of the tagantial boundary points for the droplet front and the back 
    [phif, phib]  = filmTangentenWinkel(af(j),bf(j),xm_front_top,ym_front_top,ab(j),bb(j),xm_back_top,ym_back_top);

    xpf                     = af(j)   .* cos(phif) + xm_front_top;
    ypf                     = bf(j)   .* sin(phif) + ym_front_top;
    zpf                     = H/2;
    
 
    
    phi                     = linspace(dphi*(phif/Nx1),phif,Nx1);
    X(i,1   :Nx1)           = af(j) .* cos(phi) + xm_front_top;
    Y(i,1   :Nx1)           = bf(j) .* sin(phi) + ym_front_top;
    Z(i,1   :Nx1)           = ones(1,Nx1).* (zpf);

    xpb                     = ab(j)    .* cos(phib) + xm_back_top;
    ypb                     = bb(j)    .* sin(phib) + ym_back_top;
    zpb                     = H/2; 
    
    
    
    X(i,Nx1+1  :Nx2)        = xpf + linspace((1/(Nx1))/2,1-(1/(Nx1))/2,Nx2-Nx1).* (xpb-xpf);
    m                       = (ypb-ypf)/(xpb-xpf);
    b                       = (xpb*ypf-xpf*ypb)/(xpb-xpf);
    Y(i,Nx1+1  :Nx2)        = m .* X(i,Nx1+1  :Nx2) + b;
    Z(i,Nx1+1  :Nx2)        = ones(1,Nx2-Nx1).* (zpf);


    
    phi                     = linspace(phib,1*pi-dphi*((1*pi-phib)/(Nx1)),Nx1);
    X(i,Nx2+1  :  end)  	= ab(j) .* cos(phi) + xm_back_top;
    Y(i,Nx2+1  :  end)   	= bb(j) .* sin(phi) + ym_back_top;
    Z(i,Nx2+1  :  end)   	= ones(1,Nx1).* (zpf);

end

%% z componenet of the free surface 

z               = linspace(0,(1-(1/(Nz))/2)*H/2,Nz);


%% gutter guiding line front

zg_hv                       = z;

alpha1                      = zeros(size(zg_hv));
alpha1(zg_hv>=(H/2-Rgf))    = asin((zg_hv(zg_hv>=(H/2-Rgf))-(H/2-Rgf))./Rgf);

xg_hv                       = xgf -    (a_front_left)/2 .*(-1).*(1 +cos(zg_hv./(H/2).*pi));
xg_hv(zg_hv<=0)             = xmgf + a_front_left;
yg_hv                       = ymgf + Rgf .* cos(alpha1);



%% gutter guiding line back

zg_hh                       = z;

alpha2                      = zeros(size(zg_hh));
alpha2(zg_hh>=(H/2-Rgb))    = asin((zg_hh(zg_hh>=(H/2-Rgb))-(H/2-Rgb))./Rgb);

xg_hh                       = xmgb -    (a_back_left)/2 .*(+1).*(1 +cos(zg_hh./(H/2).*pi));
xg_hh(zg_hh<=0)             = xmgb - a_back_left;
yg_hh                       = ymgb + Rgb .* cos(alpha2);


%%

% cap  front

phi_zf   = asin((z-zm_front_xz)/c_front_xz);
x_zf     = xm_front_xz + a_front_xz * cos(phi_zf);

xmf     = xg_hv;
ymf     = ym_front_xz  .* ones(size(xmf));
zmf     = z;

af      = abs(abs(xmf)-abs(x_zf) );
bf      = abs(yg_hv);

% cap back

phi_zb   = asin((z-zm_back_xz)/c_back_xz);
x_zb     = xm_back_xz - a_back_xz * cos(phi_zb);

xmb     = xg_hh;
ymb     = ym_back_xz .* ones(size(xmb));
zmb     = z;

ab      = abs(abs(xmb)-abs(x_zb));
bb      = abs(yg_hh);


for i = 1: Nz
    
    j = i;
    
    % Calculation of the tagantial boundary points for the droplet front and the back 
    [phif, phib]  = filmTangentenWinkel(af(j),bf(j),xmf(j),ymf(j),ab(j),bb(j),xmb(j),ymb(j));

    xpf                     = af(j)   .* cos(phif) + xmf(j);
    ypf                     = bf(j)   .* sin(phif) + ymf(j);
    zpf                     = H/2;

    phi                     = linspace(dphi*(phif/(Nx1)),phif,Nx1);
    X(i,1        :Nx1)  	= af(j) .* cos(phi) + xmf(j);
    Y(i,1        :Nx1)   	= bf(j) .* sin(phi) + ymf(j);
    Z(i,1        :Nx1)   	= ones(1,Nx1).* (z(j));

    xpb                     = ab(j)    .* cos(phib) + xmb(j);
    ypb                     = bb(j)    .* sin(phib) + ymb(j);
    zpb                     = H/2; 
    
	X(i,Nx1+1  :Nx2)   	= xpf + linspace((1/(Nx1))/2,1-(1/(Nx1))/2,Nx2-Nx1).* (xpb-xpf);
    m                       = (ypb-ypf)/(xpb-xpf);
    b                       = (xpb*ypf-xpf*ypb)/(xpb-xpf);
    Y(i,Nx1+1  :Nx2)   	= m .* X(i,Nx1+1  :Nx2) + b;
    Z(i,Nx1+1  :Nx2)    = ones(1,Nx2-Nx1).* (z(j));

    phi                     = linspace(phib,1*pi-dphi*((1*pi-phib)/(Nx1)),Nx1);
    X(i,Nx2+1  :  end)    = ab(j) .* cos(phi) + xmb(j);
    Y(i,Nx2+1  :  end)    = bb(j) .* sin(phi) + ymb(j);
    Z(i,Nx2+1  :  end)    = ones(1,Nx1).* (z(j));

end


%% redistribution of the above calculated surface point to from the entire 3D surface

% frontal point of view - looking in direction of the negative x-axis
% numbered quaters  
% 2 | 1
% --|--
% 3 | 4
%   
SurfaceData0        = ones(2*N,2*N,3).*NaN;

%bottom left => 3
SurfaceData0(0*N+1 : 1*N, 0*N+1 : 1*N, 1) = +rot90(fliplr(X),2);
SurfaceData0(0*N+1 : 1*N, 0*N+1 : 1*N, 2) = +rot90(fliplr(Y),2);
SurfaceData0(0*N+1 : 1*N, 0*N+1 : 1*N, 3) = -rot90(fliplr(Z),2);

%bottom right => 4
SurfaceData0(1*N+1 : 2*N, 0*N+1 : 1*N, 1) = +rot90((X),2);
SurfaceData0(1*N+1 : 2*N, 0*N+1 : 1*N, 2) = -rot90((Y),2);
SurfaceData0(1*N+1 : 2*N, 0*N+1 : 1*N, 3) = -rot90((Z),2);

% top right => 1
SurfaceData0(0*N+1 : 1*N,1*N+1 : 2*N,1) = +(X);
SurfaceData0(0*N+1 : 1*N,1*N+1 : 2*N,2) = +(Y);
SurfaceData0(0*N+1 : 1*N,1*N+1 : 2*N,3) = +(Z);

% top left => 2
SurfaceData0(1*N+1 : 2*N,1*N+1 : 2*N,1) = +fliplr(X);
SurfaceData0(1*N+1 : 2*N,1*N+1 : 2*N,2) = -fliplr(Y);
SurfaceData0(1*N+1 : 2*N,1*N+1 : 2*N,3) = +fliplr(Z);



%% Calculation of the surface normals
[Nx,Ny,Nz]                      = surfnorm(X,Y,Z);

SurfaceNormal0                  = ones(2*N,2*N,3).*NaN;

SurfaceNormal0(0*N+1 : 1*N, 0*N+1 : 1*N, 1) = +rot90(fliplr(Nx),2);
SurfaceNormal0(0*N+1 : 1*N, 0*N+1 : 1*N, 2) = +rot90(fliplr(Ny),2);
SurfaceNormal0(0*N+1 : 1*N, 0*N+1 : 1*N, 3) = -rot90(fliplr(Nz),2);

SurfaceNormal0(1*N+1 : 2*N, 0*N+1 : 1*N, 1) = +rot90((Nx),2);
SurfaceNormal0(1*N+1 : 2*N, 0*N+1 : 1*N, 2) = -rot90((Ny),2);
SurfaceNormal0(1*N+1 : 2*N, 0*N+1 : 1*N, 3) = -rot90((Nz),2);

SurfaceNormal0(0*N+1 : 1*N,1*N+1 : 2*N,1) = +(Nx);
SurfaceNormal0(0*N+1 : 1*N,1*N+1 : 2*N,2) = +(Ny);
SurfaceNormal0(0*N+1 : 1*N,1*N+1 : 2*N,3) = +(Nz);

SurfaceNormal0(1*N+1 : 2*N,1*N+1 : 2*N,1) = +fliplr(Nx);
SurfaceNormal0(1*N+1 : 2*N,1*N+1 : 2*N,2) = -fliplr(Ny);
SurfaceNormal0(1*N+1 : 2*N,1*N+1 : 2*N,3) = +fliplr(Nz);




    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Joining the surface points to an entire surface in a single matrix  %%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    SurfaceData_a   = horzcat(SurfaceData0(0*N+1 : 1*N, 0*N+1 : 1*N,:),SurfaceData0(1*N+1 : 2*N, 0*N+1 : 1*N,:));
    SurfaceData_b   = horzcat(SurfaceData0(0*N+1 : 1*N, 1*N+1 : 2*N,:),SurfaceData0(1*N+1 : 2*N, 1*N+1 : 2*N,:));
    SurfaceData     = vertcat(SurfaceData_a,SurfaceData_b);
    
    SurfaceNormal_a   = horzcat(SurfaceNormal0(0*N+1 : 1*N, 0*N+1 : 1*N,:),SurfaceNormal0(1*N+1 : 2*N, 0*N+1 : 1*N,:));
    SurfaceNormal_b   = horzcat(SurfaceNormal0(0*N+1 : 1*N, 1*N+1 : 2*N,:),SurfaceNormal0(1*N+1 : 2*N, 1*N+1 : 2*N,:));
    SurfaceNormal     = vertcat(SurfaceNormal_a,SurfaceNormal_b);
    

%%

if Flag_ShowFigure == 1
    
    figure(101)
    close gcf
    figure(101)
    hold on

    plot3(x1,y1,z1,'r-')
    plot3(x2,y2,z2,'k-')
    line([xpf1 xpb2],[ypf1 ypb2] ,[z1(1) z2(1)])
    
	plot3(x3,y3,z3,'r-')
    plot3(x4,y4,z4,'k-')
    line([xpfl3 xpbl4], [y3(1) y4(1)] ,[zpfl3 zpbl4])
    
    plot3(xgf,ygf,zgf,'r-')
    plot3(xgb,ygb,zgb,'k-')
    
    plot3(x_front_xy,y_front_xy,z_front_xy,'r-')
    plot3(x_front_xz,y_front_xz,z_front_xz,'r-')
    plot3(x_back_xy,y_back_xy,z_back_xy,'k-')
    plot3(x_back_xz,y_back_xz,z_back_xz,'k-')
    
    
   ea = 0.025;
   fa = 1;


   scf5 = surf(     squeeze(SurfaceData(:,:,1)),...
                    squeeze(SurfaceData(:,:,2)),...
                    squeeze(SurfaceData(:,:,3)),...
                    'edgealpha',ea,'facealpha',fa);
   qiv5 = quiver3(  squeeze(SurfaceData(:,:,1)),...
                    squeeze(SurfaceData(:,:,2)),...
                    squeeze(SurfaceData(:,:,3)),...
                    squeeze(SurfaceNormal(:,:,1)),...
                    squeeze(SurfaceNormal(:,:,2)),...
                    squeeze(SurfaceNormal(:,:,3)),...
                    1);
   
    colormap('gray')
    
    
    hold off


    box on    
    axis equal 
    xlim([-Ld 0])
    
    ylim([-Wch/2 Wch/2])
    zlim([-Hch/2 Hch/2])
    
    view([145 20])
    
set(gcf,'Position',[9 39 944 1078])

end

if Flag_ComputeVolume == 1
    warning('off')
    [~,Vd] =    convexHull(...
                    delaunayTriangulation(...
                        reshape(SurfaceData(:,:,1),1,[])',...
                        reshape(SurfaceData(:,:,2),1,[])',...
                        reshape(SurfaceData(:,:,3),1,[])'));
    warning('on')
end



end



function [phi1, phi2] = filmTangentenWinkel(a1,b1,xm1,ym1,a2,b2,xm2,ym2)

a           = a1/a2;
b           = b1/b2;

x           = (xm1-xm2)/a1;
y           = (ym1-ym2)/b1;


exponent    = 6;

dphi        = pi/4;
phi0        = pi/4;
phi01       = linspace(phi0-dphi,phi0+dphi,10^exponent);
phi02       = atan( a/b .* tan(phi01));
value1      = cos(phi01)./sin(phi01) + x./sin(phi01) -cos(phi02)./(a.*sin(phi01)) + ...
              sin(phi01)./cos(phi01) + y./cos(phi01) -sin(phi02)./(b.*cos(phi01))      ;
[~,idx]     = min(abs(value1));

dphi        = phi01(idx)-phi01(idx-1);
phi0        = phi01(idx);
phi01       = linspace(phi0-dphi,phi0+dphi,10^exponent);
phi02       = atan( a/b .* tan(phi01));
value1      = cos(phi01)./sin(phi01) + x./sin(phi01) -cos(phi02)./(a.*sin(phi01)) + ...
              sin(phi01)./cos(phi01) + y./cos(phi01) -sin(phi02)./(b.*cos(phi01))      ;

[~,idx]     = min(abs(value1));

dphi        = phi01(idx)-phi01(idx-1);
phi0        = phi01(idx);
phi01       = linspace(phi0-dphi,phi0+dphi,10^exponent);
phi02       = atan( a/b .* tan(phi01));
value1      = cos(phi01)./sin(phi01) + x./sin(phi01) -cos(phi02)./(a.*sin(phi01)) + ...
              sin(phi01)./cos(phi01) + y./cos(phi01) -sin(phi02)./(b.*cos(phi01))      ;

[~,idx]     = min(abs(value1));

phi1        = phi01(idx);
phi2        = atan(a/b* tan(phi1));


end











































