function [kgf,kgb,d_h] = DynamicGutterRadius_EiF(h,ar,kf,kb,Ca,Re,N)
%
% FUNCTION_DynamicGutterRadius_EiF       
%
%           calculates the dynamic gutter radius of Taylor droplets 
%
%        	as pubilshed    Miener, U., Helmers, T., Lindken, R. et al. 
%                         	Exp Fluids (2019) 60: 75. 
%                         	https://doi.org/10.1007/s00348-019-2719-0
%
%
%
%   [kgf,kgb,d_h] = FUNCTION_DynamicGutterRadius_EiF(h,ar,kf,kb,Ca,Re,N)
%
% 
%   returns                 kgf     -   dimensionless front gutter radius of a Taylor droplet.
%                           kgb     -   dimensionless back  gutter radius of a Taylor droplet. 
%                           d_h     -   dimensionless film thickness (e.g. DOI:10.1002/ceat.201100643)
%
%   inuput parameters       h       -   channel heigth
%                           ar      -   aspect ratio of the microfluidic channel
%                           kf      -   front cap defromation factor 
%                           kb      -   back  cap defromation factor 
%                           Ca      -   Capillary number of the Taylor flow
%                           Re      -   Reynolds  number of the Taylor flow
%

    % dimensionless numbers:
    We              = Re*Ca;

    % reference - static - correlation for diagonal filmthickness aka gutter diameter
    % eq. 8 (Re<2000) Sobieszuk Aubin 2012 CET 35,8 p1346-1358 DOI: 10.1002/ceat.201100643
    dD_hD0      = 0.122; 
    kg0       	= ((dD_hD0)/(2^0.5-1));
    
    % correlation for diagonal filmthickness aka gutter diameter
    % Eq. 8 (Re<2000) Sobieszuk Aubin 2012 CET 35,8 p1346-1358 DOI: 10.1002/ceat.201100643
    dD_hD       = 0.122 + 1.22*Ca^(2/3)/(1+7.28*Ca^(2/3)-0.255*We^0.215); 
    DD          = dD_hD * h;  
    
    % correlation for wall film thickness 
    % eq. 6 (Re<2000) Sobieszuk Aubin 2012 CET 35,8 p1346-1358 DOI: 10.1002/ceat.201100643
    d_h         =  0.670*Ca^(2/3) / (  1+ 1.13*Ca^(2/3) + 0.504*Ca^(0.672)*Re^(0.589) - 0.352*We^(0.629)  ); 
    dd          = d_h*h;
    

    % calculation with zero-finding algorythm, due to complex equation system
    
    % variation of the gutter radii 
    KG          = linspace(0.25,0.45,N);
    % reference gutter radius static case - WITHOUT FLOW
    KG0         = ones(size(KG)).*kg0;

    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calculation of the static frontal gutter radius:    
    % geometry parameter for the static case - INDEX 0
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        ki        	= 1;
        KGI       	= KG0;
        delta     	= 0;
        
        yprime      =       h/2.*(ar-1);  
        b1          =       h/2.*  (ar-2*delta/h);
        a1          = ki .* b1;
        aprime      = a1./b1.*sqrt(b1.^2-yprime.^2);
        bprime      = h*sqrt(2)/2 * (1-2*delta/h) .* (1-2.*KGI.*(1-1/sqrt(2)));
        Ra          = aprime.^2 ./ bprime; % semi-minor axis

        xpriprime   = ki.*h/2.*(1-2*delta/h).*(1-2*KGI);
        b3          =     h/2.*((ar-2*delta/h)-2*KGI.*(1-2*delta/h));
        a3          = ki.*b3;
        ypriprime   = b3./a3.*sqrt(a3.^2-xpriprime.^2);
        bpriprime   = h./2.*(ar-2*delta/h)-ypriprime;
        apriprime   = h./2.*( 1-2*delta/h);
        
        xab         = h/2.*(1-2*delta/h).*(1-2.*KGI.*(1-1/sqrt(2)));
        yab         = bpriprime./apriprime.*sqrt(apriprime.^2-xab.^2);
        Rb          = 1./(apriprime.^4.*bpriprime.^4).*sqrt((apriprime.^4.*yab.^2+bpriprime.^4.*xab.^2).^3);
        
        Rg          = KGI.*h.*(1-2*delta/h);
    
        Ra0f        = Ra;
        Rb0f        = Rb;
        Rg0f        = Rg;
    

    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    % calculation of the frontal gutter radius:    
    % geometry parameters for the dynamic case - NO INDEX 
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        ki        	= kf;
        KGI       	= KG;
        delta      	= dd;
    
        yprime      =       h/2.*(ar-1);  
        b1          =       h/2.*  (ar-2*delta/h);
        a1          = ki .* b1;
        aprime      = a1./b1.*sqrt(b1.^2-yprime.^2);
        bprime      = h*sqrt(2)/2 * (1-2*delta/h) .* (1-2.*KGI.*(1-1/sqrt(2)));
        Ra          = aprime.^2 ./ bprime; % semi minor axis

        xpriprime   = ki.*h/2.*(1-2*delta/h).*(1-2*KGI);
        b3          =     h/2.*((ar-2*delta/h)-2*KGI.*(1-2*delta/h));
        a3          = ki.*b3;
        ypriprime   = b3./a3.*sqrt(a3.^2-xpriprime.^2);
        bpriprime   = h./2.*(ar-2*delta/h)-ypriprime;
        apriprime   = h./2.*( 1-2*delta/h);
        
        xab         = h/2.*(1-2*delta/h).*(1-2.*KGI.*(1-1/sqrt(2)));
        yab         = bpriprime./apriprime.*sqrt(apriprime.^2-xab.^2);
        Rb          = 1./(apriprime.^4.*bpriprime.^4).*sqrt((apriprime.^4.*yab.^2+bpriprime.^4.*xab.^2).^3);

        Rg          = KGI.*h.*(1-2*delta/h);
    
        Raf         = Ra;
        Rbf         = Rb;
        Rgf         = Rg;
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    % summation equation to find KG with zero-finding algorithm
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
        Term_Af_neu    	=  (1./Ra0f	- 1./Raf);
        Term_Bf_neu    	=  (1./Rb0f	- 1./Rbf);
        Term_Cf_neu    	= -(1./Rg0f	- 1./Rgf);

        REST_F          =  (    Term_Af_neu +...
                                Term_Bf_neu +...
                                Term_Cf_neu );

        REST_F_2        = abs(REST_F);

        [~, idxf]       = min(REST_F_2);
        kgf             =  KG(idxf);
    
  
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % calculation of the static back gutter radius:    
    % geometry parameter for the static case - INDEX 0
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        ki        	= 1;
        KGI        	= KG0;

        delta      	= 0;
        
        yprime      =       h/2.*(ar-1);  
        b1          =       h/2.*  (ar-2*delta/h);
        a1          = ki .* b1;
        aprime      = a1./b1.*sqrt(b1.^2-yprime.^2);
        bprime      = h*sqrt(2)/2 * (1-2*delta/h) .* (1-2.*KGI.*(1-1/sqrt(2)));
        Ra          = bprime.^2 ./ aprime; % semi-major axis

        xpriprime   = ki.*h/2.*(1-2*delta/h).*(1-2*KGI);
        b3          =     h/2.*((ar-2*delta/h)-2*KGI.*(1-2*delta/h));
        a3          = ki.*b3;
        ypriprime   = b3./a3.*sqrt(a3.^2-xpriprime.^2);
        bpriprime   = h./2.*(ar-2*delta/h)-ypriprime;
        apriprime   = h./2.*( 1-2*delta/h);
        
        xab         = h/2.*(1-2*delta/h).*(1-2.*KGI.*(1-1/sqrt(2)));
        yab         = bpriprime./apriprime.*sqrt(apriprime.^2-xab.^2);
        Rb          = 1./(apriprime.^4.*bpriprime.^4).*sqrt((apriprime.^4.*yab.^2+bpriprime.^4.*xab.^2).^3);

        Rg          = KGI.*h.*(1-2*delta/h);
    
        Ra0b        = Ra;
        Rb0b        = Rb;
        Rg0b        = Rg;
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    % calculation of the back gutter radius:    
    % geometry parameters for the dynamic case - NO INDEX 
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
        ki        	= kb;
        KGI        	= KG;

        delta     	= dd;
    
        yprime      =       h/2.*(ar-1);  
        b1          =       h/2.*  (ar-2*delta/h);
        a1          = ki .* b1;
        aprime      = a1./b1.*sqrt(b1.^2-yprime.^2);
        bprime      = h*sqrt(2)/2 * (1-2*delta/h) .* (1-2.*KGI.*(1-1/sqrt(2)));
        Ra          = bprime.^2 ./ aprime; % semi-major axis

        xpriprime   = ki.*h/2.*(1-2*delta/h).*(1-2*KGI);
        b3          =     h/2.*((ar-2*delta/h)-2*KGI.*(1-2*delta/h));
        a3          = ki.*b3;
        ypriprime   = b3./a3.*sqrt(a3.^2-xpriprime.^2);
        bpriprime   = h./2.*(ar-2*delta/h)-ypriprime;
        apriprime   = h./2.*( 1-2*delta/h);
        
        xab         = h/2.*(1-2*delta/h).*(1-2.*KGI.*(1-1/sqrt(2)));
        yab         = bpriprime./apriprime.*sqrt(apriprime.^2-xab.^2);
        Rb          = 1./(apriprime.^4.*bpriprime.^4).*sqrt((apriprime.^4.*yab.^2+bpriprime.^4.*xab.^2).^3);

        Rg          = KGI.*h.*(1-2*delta/h);
    
        Rab         = Ra;
        Rbb         = Rb;
        Rgb         = Rg;
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    % summation equation to find KG with zero-finding algorithm
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
        Term_Ab_neu     =  (1./Ra0b	- 1./Rab);
        Term_Bb_neu     =  (1./Rb0b	- 1./Rbb);
        Term_Cb_neu     = -(1./Rg0b	- 1./Rgb);

        REST_B          =  (   	Term_Ab_neu +...
                                Term_Bb_neu +...
                                Term_Cb_neu );

        REST_B_2        = abs(REST_B);

        [~, idxb]       = min(REST_B_2);
        kgb             =  KG(idxb);

end



